#include <libudev.h>
#include <fcntl.h>
#include <gtkmm.h>
#include <glibmm/main.h>
#include <libinput.h>
#include <iostream>

static int open_restricted(const char *path, int flags, void *user_data)
{
        int fd = open(path, flags);
        return fd < 0 ? -errno : fd;
}
 
static void close_restricted(int fd, void *user_data)
{
        close(fd);
}
 
const static struct libinput_interface interface = {
        .open_restricted = open_restricted,
        .close_restricted = close_restricted,
};

void loopfunction(struct libinput_event* event, struct libinput* li) {
                        libinput_dispatch(li);        

        std::cout <<"starting event loop" << std::endl;
        while ((event = libinput_get_event(li)) != NULL) {

                // handle the event here
                if (libinput_event_get_type(event)  == LIBINPUT_EVENT_DEVICE_ADDED)
                    {

                        struct libinput_device* dev = libinput_event_get_device(event);
                        if (!libinput_device_has_capability(dev, LIBINPUT_DEVICE_CAP_GESTURE) && !libinput_device_has_capability(dev, LIBINPUT_DEVICE_CAP_TOUCH) && libinput_device_has_capability(dev, LIBINPUT_DEVICE_CAP_POINTER))
                            {
                                const char * str = libinput_device_get_name(dev);
                                std::cout <<"Mouse:" << str << std::endl;
                                if (strstr(str, "TrackPoint") != NULL)
                                        std::cout << "Trackpoint found!\n";
                            }
                        else
                            std::cout <<  "other device" << libinput_device_get_name(dev) << "\n";
                    }
                else if (libinput_event_get_type(event)  == LIBINPUT_EVENT_DEVICE_REMOVED) {
                        std::cout <<  "device removed" << std::endl;
                }
                else
                    std::cout <<  "other event" << std::endl;
                libinput_event_destroy(event);
                libinput_dispatch(li);        
        }
        std::cout <<  "finished event loop" << std::endl;
}


class MyWindow : public Gtk::Window
{
public:
  MyWindow();

  struct libinput *li;
  struct libinput_event *event;
  struct udev* udev_dev = udev_new();
  int run(int argc, char**argv);
  bool lambdafunc();
};

bool MyWindow::lambdafunc() {
        loopfunction(this->event, this->li);
        return true;
}

MyWindow::MyWindow()
{
        set_title("Basic application");
        set_default_size(200, 200);
        li = libinput_udev_create_context(&interface, NULL, udev_dev);
        libinput_udev_assign_seat(li, "seat0");
        libinput_dispatch(li);

        Glib::signal_timeout().connect([this]() {
                std::cout << "called timeout" << std::endl;
                loopfunction(this->event, this->li);
                return true;
        }, 250);
}

int MyWindow::run(int argc, char**argv) {
        return 0;
}

int main(int argc, char* argv[])
{
  auto app = Gtk::Application::create("org.gtkmm.examples.base");
  MyWindow win;
  return app->run(win, argc, argv);
}
